seednumber=372398;
rand('seed',seednumber);
randn('seed',seednumber);


% load three quarterly data series.
data1 = load('GE Final Data.txt');

% take logs and first differenced to represent (non-annualized) growth rates.
QO=100*log(data1(:,1));    % world oil production
PO=100*log(data1(:,2));    % real price of oil
YI=100*log(data1(:,3));    % world industrial production index

data=[QO PO YI];

[T N] = size(data);  
% T : time period (i.e., 1947Q1 - 2008Q3)
% N = 3: number of variables.

for t=1:T-1
    Y(t,:) = data(t+1,:) - data(t,:); 
end
T=T-1;
clear data1

[T N] = size(Y);
%
NB = 30000;   % number of "burn-in" draws which are later discarded.
NG = 15000;   % number of Gibbs samplers to save
D = 1;  % intervals in Gibbs sampling to avoid autocorrelation
NTTL= NB+NG*D;  % total number of draws

L = 4;  % number of lags : VAR(4)

X11 = lagY(Y, T, L); % generate a base matrix for RHS of VAR: ie. X1 = [1, Yt-1, Yt-2, ...,Yt-L]
clear T
T = size(X11,1); % T = total # of period - # of lags
X = genX(X11, N, L, T); % re-generate X as 3-dimension matrix
%

T0 = 4*10; % use the first 40 observations as the training sample to construct prior distribution.
% i.e., 1947Q2 - 1958Q1
% for the traninig period, we estimate oil price
% uncertainty by the (Lop*2+1)-quater moving standard deviations.
Lop = 2;  % Lop determines the size of the window
%
op33 = Y(1+L-Lop:L+T0+Lop,2)';
for ii=Lop+1:T0+Lop
    X31(ii-Lop) = log(std(op33(ii-Lop:ii+Lop)));
end
X3=X31'; %X3 proxies OP uncertainty during the training period.

Y = Y(1+L:end,:)';
Y0 = Y(:,1:T0);
Y1 = Y(:,1+T0:T);
X0 = X(:,:,1:T0);
X1 = X(:,:,1+T0:T);
T = T-T0;
T_HF = length(logRV); % number of periods during which high frequency (HF) data is available. 
Ti = T - T_HF; % number of periods when HF data is not available.

% obtain OLS estimates used from the training sample.  The estimates will
% be used as parameter values of prior distributions.
DM=[zeros(103,1); ones(T-103,1)]; % the dummy for 1984Q1
X33 = genXPD(X11(1:T0,:), N, X3, DM, T0); % for Robusteness check1
[olsB,volsB,olsOmega] = surreg(Y0,X33,T0);
B=chol(olsOmega)';
invA=zeros(N,N);         
for jj=1:N
    invA(:,jj)=B(:,jj)/B(jj,jj);
end
A=inv(invA);
%
%%%%%%%%%%%%%%%%%%%%%%% SET UP THE PRIORS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (1) Prior for coefficient Bt ~ N(olsB, 4*volsB) # of element's in Bt = N*(1+N*L)

prior_meanBL =[olsB; 0];
prior_varBL = [4*volsB zeros(40,1); zeros(1,40) 1];

% (2) prior for alphas ~ N(olsalpha, prior_varA)
prior_meanA = stackA(A); % N*(N-1)/2-by-1 = 3-by-1
prior_varA = 10*diag(abs(prior_meanA)); %OLS prior

% (3) prior for log h ~ N(log h, In)
prior_logh = log(diag(B)); % N-by-1 = 3-by-1
prior_varh = eye(N);
el_varh = abs((log(diag(B)))./(diag(B).^2));
for a=1:N
    prior_varh(a,a) = el_varh(a);
end

% (4) prior for S=var(innovation of alpha) matrix ~ IW
prior_dfS1 = 2;
post_dfS1 = prior_dfS1+T-1;
prior_QS1 = prior_varA(1,1);
prior_dfS2 = 3;
post_dfS2 = prior_dfS2+T-1;
prior_QS2 = prior_varA(2:3,2:3);

% (5) prior for W = var(innovation of log h) matrix: diagonal => each
% diagonal element follows IG
prior_dfW = 1;
post_dfW = prior_dfW+T-1;
prior_QW = (0.01)^2;

% (6) Prior for rho: drift and AR(1) coefficients of SV
prior_meanrho = [0 1]';
prior_varrho = 0.05*eye(2);
