seednumber=372398;
rand('seed',seednumber);
randn('seed',seednumber);


% load three quarterly data series.
data1 = load('GE Final Data.txt');

% take logs and first differenced to represent (non-annualized) growth rates.
QO=100*log(data1(:,1));    % world oil production
PO=100*log(data1(:,2));    % real price of oil
YI=100*log(data1(:,3));    % world industrial production index

data=[QO PO YI];

[T N] = size(data);  
% T : time period (i.e., 1958Q2 - 2008Q3)
% N = 3: number of variables.

for t=1:T-1
    Y(t,:) = data(t+1,:) - data(t,:); 
end
T=T-1;
clear data1

[T N] = size(Y);
%
NB = 30000;   % number of "burn-in" draws which are later discarded.
NG = 15000;   % number of Gibbs samplers to save
D = 1;  % intervals in Gibbs sampling to avoid autocorrelation
NTTL= NB+NG*D;  % total number of draws

L = 4;  % number of lags : VAR(4)

X1 = lagY(Y, T, L); % generate a base matrix for RHS of VAR: ie. X1 = [1, Yt-1, Yt-2, ...,Yt-L]
clear T
T = size(X1,1); % T = total # of period - # of lags
X = genX(X1, N, L, T); % re-generate X as 3-dimension matrix
%

T0 = 4*10; % use the first 40 observations as the training sample to construct prior distribution.

sb=(144-1)+4*stbreak; % t=144 is 1984Q1
Y1 = Y(1+L:sb,:)';
Y01 = Y1(:,1:T0);
Y11 = Y1(:,1+T0:end);
X01 = X(:,:,1:T0);
X11 = X(:,:,1+T0:sb);

Y2 = Y(1+sb-T0:end,:)';
Y02 = Y2(:,1:T0);
Y12 = Y2(:,1+T0:end);
X02 = X(:,:,sb+1-T0:sb);
X12 = X(:,:,sb+1:end);

T1 = sb-T0-L;
T2 = T-sb-T0-L;
T_HF = T2; % number of periods during which high frequency (HF) data is available. 
Ti = 0; % number of periods when HF data is not available.
logRV=logRV(end-T_HF+1:end,:);
Sigma=Sigma(end-T_HF+1:end,:);

% for the traninig period, we estimate oil price
% uncertainty by the (Lop*2+1)-quater moving standard deviations.
Lop = 2;  % Lop determines the size of the window

op31 = Y(1+L-Lop:L+T0+Lop,2)';
for ii=Lop+1:T0+Lop
    X31(ii-Lop) = log(std(op31(ii-Lop:ii+Lop)));
end
X31=X31'; %X31 proxies OP uncertainty during the first training period.

op32 = Y(sb+1+L-Lop:sb+L+T0+Lop,2)';
for ii=Lop+1:T0+Lop
    X32(ii-Lop) = log(std(op32(ii-Lop:ii+Lop)));
end
X32=X32'; %X3 proxies OP uncertainty during the second training period.


%
% obtain OLS estimates used from the training sample.  The estimates will
% be used as parameter values of prior distributions.
X21 = genXP(X1(1:T0,:), N, X31, T0);

[olsB1,volsB1,olsOmega1] = surreg(Y01,X21,T0);
B1=chol(olsOmega1)';
invA1=zeros(N,N);         
for jj=1:N
    invA1(:,jj)=B1(:,jj)/B1(jj,jj);
end
A11=inv(invA1);

X22 = genXP(X1(sb+1:sb+T0,:), N, X32, T0);

[olsB2,volsB2,olsOmega2] = surreg(Y02,X22,T0);
B2=chol(olsOmega2)';
invA2=zeros(N,N);         
for jj=1:N
    invA2(:,jj)=B2(:,jj)/B2(jj,jj);
end
A12=inv(invA2);
%
%%%%%%%%%%%%%% priors for the first subsample %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (1) Prior for coefficient Bt ~ N(olsB, 4*volsB) # of element's in Bt = N*(1+N*L)
prior_meanBL1 = olsB1; 
prior_varBL1 = 4*volsB1;

% (2) prior for alphas ~ N(olsalpha, prior_varA)
prior_meanA11 = stackA(A11); % N*(N-1)/2-by-1 = 3-by-1
prior_varA11 = 10*diag(abs(prior_meanA11)); %OLS prior

% (3) prior for log h ~ N(log h, In)
prior_logh1 = log(diag(B1)); % N-by-1 = 3-by-1
prior_varh1 = eye(N);
el_varh1 = abs((log(diag(B1)))./(diag(B1).^2));
for a=1:N
    prior_varh1(a,a) = el_varh1(a);
end

% (4) prior for S=var(innovation of alpha) matrix ~ IW
prior_dfS11 = 2;
post_dfS11 = prior_dfS11+T1-1;
prior_QS11 = prior_varA11(1,1);
prior_dfS21 = 3;
post_dfS21 = prior_dfS21+T1-1;
prior_QS21 = prior_varA11(2:3,2:3);

% (5) prior for W = var(innovation of log h) matrix: diagonal => each
% diagonal element follows IG
prior_dfW1 = 1;
post_dfW1 = prior_dfW1+T1-1;
prior_QW1 = (0.01)^2;

% (6) Prior for rho: drift and AR(1) coefficients of SV
prior_meanrho1 = [0 1]';
prior_varrho1 = 0.05*eye(2);

%%%%%%%%%%%%%% priors for the second subsample %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (1) Prior for coefficient Bt ~ N(olsB, 4*volsB) # of element's in Bt = N*(1+N*L)
prior_meanBL2 = olsB2; 
prior_varBL2 = 4*volsB2;

% (2) prior for alphas ~ N(olsalpha, prior_varA)
prior_meanA22 = stackA(A12); % N*(N-1)/2-by-1 = 3-by-1
prior_varA22 = 10*diag(abs(prior_meanA22)); %OLS prior

% (3) prior for log h ~ N(log h, In)
prior_logh2 = log(diag(B2)); % N-by-1 = 3-by-1
prior_varh2 = eye(N);
el_varh2 = abs((log(diag(B2)))./(diag(B2).^2));
for a=1:N
    prior_varh2(a,a) = el_varh2(a);
end

% (4) prior for S=var(innovation of alpha) matrix ~ IW
prior_dfS12 = 2;
post_dfS12 = prior_dfS12+T2-1;
prior_QS12 = prior_varA22(1,1);
prior_dfS22 = 3;
post_dfS22 = prior_dfS22+T2-1;
prior_QS22 = prior_varA22(2:3,2:3);

% (5) prior for W = var(innovation of log h) matrix: diagonal => each
% diagonal element follows IG
prior_dfW2 = 1;
post_dfW2 = prior_dfW2+T2-1;
prior_QW2 = (0.01)^2;

% (6) Prior for rho: drift and AR(1) coefficients of SV
prior_meanrho2 = [0 1]';
prior_varrho2 = 0.05*eye(2);
