%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is to draw time-varying volatilities from the conditional
% distributions given as h | A, logIV, Sigma, Y which incorporates the
% information obtained from the high frequency data.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [UPDATE :112910] The H matrix in Kalman filter for oil price uncertainty
% is "time-varying" in order to combine the information from the high
% frequency data.  That is, the dimension of H matrix increases when the HF
% data becomes available (1983~).  

% With regard to oil price uncertainty, the following applies.
% The state space representation based on this code is the following:
% measurement eqn :    y_t* = 2h_t + e_t,       e_t ~ logChi^2(1)
%                   logIV_t = h_t-1 + u_t,      u_t ~ N(0, Sigma/M)
% state eqns : h_t = h_t-1 + eta_t,      eta_t ~ N(0,w)


% Thus, from conventional notations, i.e. 
% measurement eqn : y_t = H' xi_t + w_t,  w_t ~ N(0,R)
% state eqn : xi_t = F xi_t-1 + v_t,     v_t ~ N(0,Q),
% 
% y_t = [ystar_t logIV_t]; H = [2 1]; R :=[u2_s 0; 0 Sigma/M];
% xi_t = h_t;  F = [1];  v_t = eta_t;  Q = Wdraw(2,2)

%%%%%%%%%%%%%%%%%%%%%%%% (1) Draw h_1t's first.%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ystar1 = e(:,1).^2; 
% Following Primiceri (2005), transform to the log-scale 
% but also add the 'offset constant'=0.001 
yss1 = log(ystar1 + 0.001);

vart1 = zeros(T,1);
ys1 = zeros(T,1);
for i = 1:T
     imix = statedraw(i,1);
     vart1(i) = u2_s(imix);
     ys1(i) = yss1(i) - m_s(imix) + 1.2704;
end

% draw oil production uncertainty.
%lhdraw1 = mixednormalh(ys1',vart1,Wdraw(1,1),1,1,T,prior_logh(1),prior_varh(1,1),rho1);
F1 = [1 0; int1 rho1];
Q=[zeros(1,2); 0 Wdraw(1,1)];
lhdraw1 = kfh1([0 2]', Q, F1, vart1, ys1,[0 prior_logh(1)]',[zeros(1,2);0 prior_varh(1,1)], 2);
 % Next draw state (chi square approximation mixture component) conditional on Sigtdraw
 % This is used to update at the next step the log-volatilities Sigtdraw
%
 for i = 1:T
     for j = 1:numel(m_s)
            % temp1= (1/sqrt(2*pi*u2_s(j)))*exp(-.5*(((yss(i,jj) - lhdraw1(jj,i) - m_s(j) + 1.2704)^2)/u2_s(j)));
            temp1= (1/sqrt(2*pi*u2_s(j)))*exp(-.5*(((yss1(i) - 2*lhdraw1(i) - m_s(j) + 1.2704)^2)/u2_s(j)));
            prw(j,1) = q_s(j,1)*temp1;
     end
     prw = prw./sum(prw);
     cprw = cumsum(prw);
     trand = rand(1,1);
     if trand < cprw(1,1), imix=1;
     elseif trand < cprw(2,1), imix=2;
     elseif trand < cprw(3,1), imix=3;
     elseif trand < cprw(4,1), imix=4;
     elseif trand < cprw(5,1), imix=5;
     elseif trand < cprw(6,1), imix=6;
     else imix=7; 
     end
            % statedraw T+1 * N matrix
     statedraw(i,1)=imix;  % this is a draw of the mixture component index
end
tempH = zeros(3, T);
tempH(1,:) = lhdraw1;

%%%%%%%%%%%%%%%%%%%%%%%%%%% (2) Draw h_2t's. %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for t=1:T
   U23(:,t) = AA(3,:,t)*(Y1(:,t) - X1(:,:,t)'*Bdraw) ;
end

ystar2 = e(:,2).^2;
yss2 = log(ystar2(:,1) + 0.001);
 
vart22 = zeros(T_HF*3,3); % vart2 := R
vart21 = zeros(Ti*2,2);
ys22 = zeros(T_HF*3,1);
ys21 = zeros(Ti*2,1); 
for i = 1:T
     imix = statedraw(i,2);
     if i <= Ti
         vart21(2*(i-1)+1,1) = u2_s(imix);
         vart21(2*(i-1)+2,2) = Hdraw(3,i)^2;
         ys21(2*(i-1)+1) = yss2(i) - m_s(imix) + 1.2704;
         ys21(2*i) = U23(i);
     elseif i > Ti
         ii = i - Ti;
         vart22((ii-1)*3+1,1) = u2_s(imix);
         vart22(3*(ii-1)+2,2) = Hdraw(3,i)^2;  
         vart22(3*ii,3) = ppdraw*Sigma(ii);   
         ys22((ii-1)*3+1) = yss2(i) - m_s(imix) + 1.2704;
         ys22((ii-1)*3+2) = U23(i);
         ys22(3*ii) = logRV(ii);
     end
end

% Draw oil price uncertainty.
RV_oilh;
tempH(2,:) = lhdraw2;

for i = 1:T
     for j = 1:numel(m_s)
            temp1= (1/sqrt(2*pi*u2_s(j)))*exp(-.5*(((yss2(i) - 2*lhdraw2(i) - m_s(j) + 1.2704)^2)/u2_s(j)));
            prw(j,1) = q_s(j,1)*temp1;
     end
     prw = prw./sum(prw);
     cprw = cumsum(prw);
     trand = rand(1,1);
     if trand < cprw(1,1), imix=1;
     elseif trand < cprw(2,1), imix=2;
     elseif trand < cprw(3,1), imix=3;
     elseif trand < cprw(4,1), imix=4;
     elseif trand < cprw(5,1), imix=5;
     elseif trand < cprw(6,1), imix=6;
     else imix=7; 
     end
            % statedraw T+1 * N matrix
     statedraw(i,2)=imix;  % this is a draw of the mixture component index
end

%%%%%%%%%%%%%%%%%%%%%%%%%%% (3) Draw h_3t's. %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Next draw state (chi square approximation mixture component) conditional
% on h_1t and h2t. 

tempHH = genHH(tempH', N, T);
ystar3 = zeros(1,T);
for t=1:T
    ystar3(:,t) = (AA(3,:,t)*(Y1(:,t) - X1(:,:,t)'*Bdraw -tempHH(:,:,t)'*LL))^2;
end
% Transform to the log-scale but also add the 'offset constant' 
 yss3 = log(ystar3' + 0.001);
 
 vart3 = zeros(T,1);
 yss13 = zeros(T,1);
    for i = 1:T
            imix = statedraw(i,3);
            vart3(i,:) = u2_s(imix);
            yss13(i) = yss3(i) - m_s(imix) + 1.2704;
    end

F1 = [1 0; int3 rho3];
Q=[zeros(1,2); 0 Wdraw(3,3)];
lhdraw3 = kfh1([0 2]', Q, F1, vart3, yss13,[1 prior_logh(3)]',[zeros(1,2);0 prior_varh(3,3)], 2);
 
for i = 1:T
    for j = 1:numel(m_s)
                temp1= (1/sqrt(2*pi*u2_s(j)))*exp(-.5*(((yss3(i) - 2*lhdraw3(i) - m_s(j) + 1.2704)^2)/u2_s(j)));
                prw(j,1) = q_s(j,1)*temp1;
     end
            prw = prw./sum(prw);
            cprw = cumsum(prw);
            trand = rand(1,1);
    if trand < cprw(1,1); imix=1;
    elseif trand < cprw(2,1), imix=2;
    elseif trand < cprw(3,1), imix=3;
    elseif trand < cprw(4,1), imix=4;
    elseif trand < cprw(5,1), imix=5;
    elseif trand < cprw(6,1), imix=6;
    else imix=7; 
    end
    statedraw(i,3)=imix;  % this is a draw of the mixture component index
end

%%%%%%%%%%%%%%%%%%%%%% Put together the draws %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

lhdraw = [lhdraw1; lhdraw2; lhdraw3];
Hdraw = zeros(3,T);
 for j = 1:N
     Hdraw(j,:) = exp(lhdraw(j,:));
 end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Draw W, the covariance of SIGMA(t) (from iWishart)
    % Get first differences of Sigtdraw to compute the SSE
    rd=repmat(rhodraw',T-1,1);
    intd=repmat(intdraw',T-1,1);
    Sigttemp = lhdraw(:,2:end)' -intd - rd.*lhdraw(:,1:end-1)';

    sse_2 = zeros(N,1);
    Wdraw = zeros(N,1);

    for ii=1:N
        %sse_2(ii) = Sigttemp(:,ii)'*Sigttemp(:,ii)+prior_QW(ii,ii);
        sse_2(ii) = Sigttemp(:,ii)'*Sigttemp(:,ii)+prior_QW;
        %z = chol(inv(sse_2(ii)))'*randn(1,post_dfW);
        z = randn(post_dfW,1);
        x = z'*z;
        Wdraw(ii,ii) = sse_2(ii)/x;
    end
    
%%%%%%%%%%%  Draw F %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for ii=1:N
    Rxox = zeros(2,2);
    Rxoy = zeros(2,1);
    yh = lhdraw(ii,2:end)'; 
    xh = [ones(T-1,1) lhdraw(ii,1:end-1)']; 
    for t = 1:T-1
        Rxox = Rxox + xh(t,:)'*inv(Wdraw(ii,ii))*xh(t,:);
        Rxoy = Rxoy + xh(t,:)'*inv(Wdraw(ii,ii))*yh(t);
    end
    %post_varrho = inv(inv(prior_varrho(ii,ii)) + xh'*inv(Wdraw(ii,ii))*xh);
    post_varrho = inv(inv(prior_varrho)+Rxox);
    post_meanrho = post_varrho*(prior_varrho\prior_meanrho + Rxoy);
    %rhohat = xh'*inv(Wdraw(ii,ii))*yh;
    %post_meanrho = post_varrho*(rhohat+inv(prior_varrho(ii,ii))*prior_meanrho(ii));
    intrho(ii,:) = (post_meanrho + chol(post_varrho)'*randn(2,1))';
end
intdraw = intrho(:,1);
rhodraw = intrho(:,2);
int1 = intdraw(1);
int2 = intdraw(2);
int3 = intdraw(3);
rho1 = rhodraw(1);
rho2 = rhodraw(2);
rho3 = rhodraw(3);
   