% This code transforms a vector of the daily oil price into cells
% and then constructs vectors of quarterly RV, IQ and Sigmas in order to estimate
% quarterly Realized Volatlity.

% The transformation is to follow Dobrev and Szerszen (2010).

data11=load('HFoilpriceGE.txt');

X=100*log(data11(:,1)); 
month=data11(:,2);
year=data11(:,3);
X = [month year X]; % I.e., X=[month year price].

N = length(X);
month=X(:,1);
year=X(:,2);
%
for i=1:N
    if month(i) <= 3 
        quarter(i) = 1;
    elseif month(i)>3 && month(i) <=6
        quarter(i) = 2;
    elseif month(i)>6 && month(i) <=9
        quarter(i) = 3;
    elseif month(i)>9 && month(i) <=12
        quarter(i) = 4;
    end
    quarter=quarter';
end

data = cell(year(N), 4); % Each cell contains daily oil prices of a specific year/quarter.

for i=1:N
    for j=1:4
        if quarter(i) == j;
            data{year(i),j} = [data{year(i),j}; X(i,3)];
        end
    end
end

y = max(year);
n = (y-1)*4+quarter(end);

% Set spaces aside for RV, IQ, and Sigma vectors.
RV = zeros(n,1);
logRV = zeros(n,1);
IQ = zeros(n,1);
Sigma = zeros(n,1);
nu=2.96;


for ii=1:y
    for j=1:4
        M=length(data{ii,j});
        if M == 0
            break
        end
        YY=data{ii,j};
        for jj=2:M
            Y(jj-1) = abs(YY(jj) - YY(jj-1));
        end            
        for i=2:M-2 % subtract 2 since length(Y) = length(YY)-1
            RV(4*(ii-1)+j,1) = RV(4*(ii-1)+j,1) + (pi/(6-4*sqrt(3)+pi))*(M/(M-2))*(median([Y(i-1) Y(i) Y(i+1)]))^2;
            IQ(4*(ii-1)+j,1) = IQ(4*(ii-1)+j,1) + (3*pi*M/(9*pi+72-52*sqrt(3)))*(M/(M-2))*(median([Y(i-1) Y(i) Y(i+1)]))^4;
        end
        Sigma(4*(ii-1)+j,1)=nu*IQ(4*(ii-1)+j,1)/(M*RV(4*(ii-1)+j,1)^2);
        logRV(4*(ii-1)+j,1) = log((RV(4*(ii-1)+j,1)));
    end
end

clear X year quarter month N data data11 y n RV IQ nu M YY Y 




        